/*
  sf_laguerre.c
  Ruby/GSL: Ruby extension library for GSL (GNU Scientific Library)
    (C) Copyright 2001-2004 by Yoshiki Tsunesada

  Ruby/GSL is free software: you can redistribute it and/or modify it
  under the terms of the GNU General Public License.
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY.
*/

#include "rb_gsl_sf.h"

static VALUE rb_gsl_sf_laguerre_1(VALUE obj, VALUE a, VALUE x)
{
  return rb_gsl_sf_eval_double_double(gsl_sf_laguerre_1, a, x);
}

static VALUE rb_gsl_sf_laguerre_1_e(VALUE obj, VALUE a, VALUE x)
{
  return rb_gsl_sf_eval_e_double2(gsl_sf_laguerre_1_e, a, x);
}

static VALUE rb_gsl_sf_laguerre_2(VALUE obj, VALUE a, VALUE x)
{
  return rb_gsl_sf_eval_double_double(gsl_sf_laguerre_2, a, x);
}

static VALUE rb_gsl_sf_laguerre_2_e(VALUE obj, VALUE a, VALUE x)
{
  return rb_gsl_sf_eval_e_double2(gsl_sf_laguerre_2_e, a, x);
}

static VALUE rb_gsl_sf_laguerre_3(VALUE obj, VALUE a, VALUE x)
{
  return rb_gsl_sf_eval_double_double(gsl_sf_laguerre_3, a, x);
}

static VALUE rb_gsl_sf_laguerre_3_e(VALUE obj, VALUE a, VALUE x)
{
  return rb_gsl_sf_eval_e_double2(gsl_sf_laguerre_3_e, a, x);
}

static VALUE rb_gsl_sf_laguerre_n(VALUE obj, VALUE n, VALUE a, VALUE x)
{
  return rb_gsl_sf_eval_int_double_double(gsl_sf_laguerre_n, n, a, x);
}

static VALUE rb_gsl_sf_laguerre_n_e(VALUE obj, VALUE n, VALUE a, VALUE x)
{
  gsl_sf_result *rslt = NULL;
  VALUE v;
  int status;
  CHECK_FIXNUM(n);
  Need_Float(a); Need_Float(x);
  v = Data_Make_Struct(cgsl_sf_result, gsl_sf_result, 0, free, rslt);
  status = gsl_sf_laguerre_n_e(FIX2INT(n), NUM2DBL(a), NUM2DBL(x), rslt);
  return v;
}

void Init_gsl_sf_laguerre()
{
  VALUE mgsl_sf_laguerre;

  rb_define_module_function(mgsl_sf, "laguerre_1",  rb_gsl_sf_laguerre_1, 2);
  rb_define_module_function(mgsl_sf, "laguerre_1_e",  rb_gsl_sf_laguerre_1_e, 2);
  rb_define_module_function(mgsl_sf, "laguerre_2",  rb_gsl_sf_laguerre_2, 2);
  rb_define_module_function(mgsl_sf, "laguerre_2_e",  rb_gsl_sf_laguerre_2_e, 2);
  rb_define_module_function(mgsl_sf, "laguerre_3",  rb_gsl_sf_laguerre_3, 2);
  rb_define_module_function(mgsl_sf, "laguerre_3_e",  rb_gsl_sf_laguerre_3_e, 2);
  rb_define_module_function(mgsl_sf, "laguerre_n",  rb_gsl_sf_laguerre_n, 3);
  rb_define_module_function(mgsl_sf, "laguerre_n_e",  rb_gsl_sf_laguerre_n_e, 3);

  mgsl_sf_laguerre = rb_define_module_under(mgsl_sf, "Laguerre");
  rb_define_module_function(mgsl_sf_laguerre, "one",  rb_gsl_sf_laguerre_1, 2);
  rb_define_module_function(mgsl_sf_laguerre, "one_e",  rb_gsl_sf_laguerre_1_e, 2);
  rb_define_module_function(mgsl_sf_laguerre, "two",  rb_gsl_sf_laguerre_2, 2);
  rb_define_module_function(mgsl_sf_laguerre, "two_e",  rb_gsl_sf_laguerre_2_e, 2);
  rb_define_module_function(mgsl_sf_laguerre, "three_3",  rb_gsl_sf_laguerre_3, 2);
  rb_define_module_function(mgsl_sf_laguerre, "three_e",  rb_gsl_sf_laguerre_3_e, 2);
  rb_define_module_function(mgsl_sf_laguerre, "n",  rb_gsl_sf_laguerre_n, 3);
  rb_define_module_function(mgsl_sf_laguerre, "n_e",  rb_gsl_sf_laguerre_n_e, 3);

}
